//--------------------------------------------------------------------------
//Manages the construction of structures.
//--------------------------------------------------------------------------

//What and were the truck is going to build
function buildTicket(tstructure, ttruck, tx, ty)
{
  this.structure = tstructure;
  this.truck = ttruck;
  this.x = tx;
  this.y = ty;
}

//--------------------------------------------------------------------------
//buildAt - Builds 'structure' using the nearest idle truck at place 'x', 'y'. If 'pick' is true, picks a location near 'x', 'y'. safe = i kow its safe.
//--------------------------------------------------------------------------
function buildAt(structure, x, y, pick, safe)
{
  var structureCost = 100; //Most structures costs 100 power. Check construction cost.
  if(structure == "A0ResourceExtractor" || structure == "A0PowMod1")
  {
    structureCost = 0;
  }
  else if(structure == "A0PowerGenerator")
  {
    structureCost = 50;
  }
  else if(structure == "Sys-SensoTowerWS")
  {
    structureCost = 250;
  }
  else if(structure == "A0LasSatCommand" || structure == "A0Sat-linkCentre")
  {
    structureCost = 1000;
  }
  if(structureCost > temp.myPower)
  {
    storage.TaskPower = structureCost + 20;
    return false;
  }
  if(temp.trucksIdle.length > 0)
  {
    var truck = getIdleTruck(x, y);
    
    function orderTruck(s)
    {
      var pX = x;
      var pY = y;
      if(pick)
      {
        var pos = pickStructLocation(truck, s, x, y);
        if(pos)
        {
          pX = pos.x;
          pY = pos.y;
        }
      }
      if(safe || safeDest(me, pX, pY))
      {
        if(orderDroidBuild(truck, DORDER_BUILD, s, pX, pY))
        {
          temp.myPower -= structureCost;
          
          storage.buildTickets.push(new buildTicket(s, truck, pX, pY));
          return true;
        }
      }
      return false;
    }
    
    if(Array.isArray(structure))
    {
      for(var a = 0; a < structure.length; a++)
      {
        if(isStructureAvailable(structure[a]))
        {
          return orderTruck(structure[a]);
        }
      }
    }
    else
    {
      return orderTruck(structure);
    }
  }
  return false;
}
//--------------------------------------------------------------------------
// Get a idle truck near some point.
//--------------------------------------------------------------------------
function getIdleTruck(x, y)
{
  return temp.trucksIdle.splice(indexOfNearest(temp.trucksIdle, x, y), 1)[0];
}
//--------------------------------------------------------------------------
//Demolish a structure using the nearest truck to it.
//--------------------------------------------------------------------------
function DemolishIt(it)
{
  if(temp.trucksIdle.length > 0)
  {
    var truck = getIdleTruck(it.x, it.y);
    return !orderDroidObj(truck, DORDER_DEMOLISH, it);
  }
  return true;
}
//--------------------------------------------------------------------------
//Find on these[] which one is the nearest of x, y.
//--------------------------------------------------------------------------
function indexOfNearest(these, x, y)
{
  var nearest = Infinity;
  var index = 0;
  for(var a = 0; a < these.length; a++)
  {
    var distance = distBetweenTwoPoints(these[a].x, these[a].y, x, y);
    if(distance < nearest)
    {
      nearest = distance;
      index = a;
    }
  }
  return index;
}
//--------------------------------------------------------------------------
//Used to avoid building derricks on friends oil. If this ally is still alive.
//--------------------------------------------------------------------------
function isNearFriendlyBase(oilR)
{
  var checkDistance = 20;
  for(var a = 0; a < playerData.length; a++)
  {
    if(a != me && allianceExistsBetween(a, me))
    {
      var distance = distBetweenTwoPoints(startPositions[a].x, startPositions[a].y, oilR.x, oilR.y);
      if(distance < checkDistance)
      {
        var friends = enumRange(oilR.x, oilR.y, checkDistance, ALLIES, false).filter(function(obj){  return obj.player == a; })
        if(friends.length > 0){
          return true;
        }
      }
    }
  }
  return false;
}
//--------------------------------------------------------------------------
//Build oil derricks if have more generators than oil.
//--------------------------------------------------------------------------
function oilHunt()
{
  if(temp.generators.length > 0 && temp.trucksIdle.length > 0)
  {
    var oilResources = checkOilResource();
    if(oilResources.length > 0)
    {
      oilResources = oilResources.filter(function (f){ return !isNearFriendlyBase(f); });  //ignore friends oil.
      if(oilResources.length > 0)
      {
        var tickets = storage.buildTickets.filter(function (t){ return t.structure == "A0ResourceExtractor"; }); //trucks busy with oil.
        if(tickets.length > 0) //Do not send another truck to build a derrick if you done it already.
        {
          oilResources = oilResources.filter
          (
            function (f)
            { 
              //debug("AI[" + me + "] oil.x = (" + f.x + ", " + f.y + ")");
              for(var a = 0; a < tickets.length; a++)
              {
                if(f.x == tickets[a].x && f.y == tickets[a].y)
                {
                  return false;
                }
              }
              return true;
            }
          )
        }
        if(oilResources.length > 0)
        {
          var trucksForOil = Math.ceil(temp.trucks.length*0.4) - tickets.length;
          //debug("AI[" + me + "] trucks " + Math.ceil(temp.trucks.length*0.4) + "/" + temp.trucks.length + " tickets " + tickets.length);
          if(trucksForOil > 0)
          {
            oilResources.sort(sort_NearestBase);
            for(var a = 0; a < oilResources.length && a < trucksForOil; a++)
            {
              buildAt("A0ResourceExtractor", oilResources[a].x, oilResources[a].y, false, distBetweenTwoPoints(oilResources[a].x, oilResources[a].y, startPositions[me].x, startPositions[me].y) < storage.baseRange);
            }
          }
        }
      }
    }
  }
}
//--------------------------------------------------------------------------
//Stops trucks that are going to build derricks at features that does not exists anymore.
//--------------------------------------------------------------------------
function oilGiveUp()
{
  var tickets = storage.buildTickets.filter(function (t){ return t.structure == "A0ResourceExtractor"; });
  if(tickets.length > 0)
  {
    var oilResources = checkOilResource();
    next:for(var a = 0; a < tickets.length; a++)
    {
      var abort = oilResources.length == 0;
      
      if(!abort)
      {
        for(var b = 0; b < oilResources.length; b++)
        {
          if(oilResources[b].x == tickets[a].x && oilResources[b].y == tickets[a].y)
          {
            continue next;
          }
        }
        abort = true;
      }
      
      if(abort)
      {
        orderDroid(tickets[a].truck, DORDER_STOP);
      }
    }
  }
}
//--------------------------------------------------------------------------
//Build the base structures.
//--------------------------------------------------------------------------
function buildBase()
{
  temp.BaseMan.run();
  
  if(temp.trucksIdle.length > 0)//Attempt to finish unfinished buildings.
  {
    unfinished = temp.structuresList.filter(function (s){ return s.status == BEING_BUILT; });
    if(unfinished.length > 0)
    {
      var truck = getIdleTruck(unfinished[0].x, unfinished[0].y);
      orderDroidObj(truck, DORDER_HELPBUILD, unfinished[0]);
      return false;
    }
  }
  return temp.myPower < 10;
}
//--------------------------------------------------------------------------
//Build power generators at base.
//--------------------------------------------------------------------------
function buildBasePower()
{
  if(isStructureAvailable("A0PowerGenerator")) //Can build more generators?.
  { 
    if(temp.generators.length < 1 || (((temp.generators.length < (temp.oilDerricks.length/4 + 1) && temp.factories.length > 0)) && temp.labs.length > 0)) //Do i have 1 generator for each 4 oil derricks?  
    {
      if(!buildAtBase("A0PowerGenerator"))
      {
        return true;
      }  
    }
  }
  buildModule(temp.generators, "A0PowMod1");
  return false;
}
//--------------------------------------------------------------------------
//Build the base structures.
//--------------------------------------------------------------------------
function buildBaseTech()
{
  if(temp.generators.length > 0 && temp.factories.length > 0)
  {
    if(isStructureAvailable("A0CommandCentre") && temp.labs.length > 0) //Build Command Center if i have at least 1 research lab.
    {
    	if(!buildAtBase("A0CommandCentre"))
      {
        return true;
      }
    }
    if(isStructureAvailable("A0ResearchFacility")) //buil research facilities.
    {
      if(!buildAtBase("A0ResearchFacility"))
      { 
        return true;
      }
    }
    buildModule(temp.labs, "A0ResearchModule1");
    if(isStructureAvailable("A0LasSatCommand")) //Laser Sat
    {
      if(!buildAtBase("A0LasSatCommand"))
      {
        return true;
      }
    }
    if(isStructureAvailable("A0Sat-linkCentre")) //Sat Uplink
    {
      if(!buildAtBase("A0Sat-linkCentre"))
      {
        return true;
      }
    }
  }
  return false;
}
//--------------------------------------------------------------------------
//Build the base structures.
//--------------------------------------------------------------------------
function buildBaseFactories()
{
  if(temp.generators.length > 0 && temp.oilDerricks.length > 0)
  {
    if(isStructureAvailable("A0LightFactory"))
    {
      if(temp.factories.length < 1 || (temp.factories.length < 2 && temp.labs.length > 0) || (temp.myPower > temp.factories.length*400 && temp.labs.length > 0 && storage.useTanks)) //Build at least 1 factory. Build more if have oil income.
      {
        if(!buildAtBase("A0LightFactory"))
        {
          return true;
        }
      }
    }
    buildModule(temp.factories, "A0FacMod1");
    if(storage.heavyVtols)
    {
      buildModule(temp.vtolFactories, "A0FacMod1");
    }
    if(isStructureAvailable("A0VtolPad")) //VTOL Rearming Pad
    {
      if(temp.vtolFactories.length > 0 && temp.vtolPads.length < storage.vtols.max) //Build a pad for each vtol
      {
        if(!buildAtBase("A0VtolPad"))
        {
          return true;
        }
      }
    }
    if(isStructureAvailable("A0VTolFactory1")) //VTOL factory
    {
      if(temp.vtolFactories.length < temp.factories.length)
      {
        if(!buildAtBase("A0VTolFactory1"))
        {
          return true;
        }
      }
    }
  }
  return false;
}
//--------------------------------------------------------------------------
// Build structures at Base
//--------------------------------------------------------------------------
function buildAtBase(structure){
  return buildAt(structure, startPositions[me].x, startPositions[me].y, true, true);
}
//--------------------------------------------------------------------------
//Build modules
//--------------------------------------------------------------------------
function buildModule(structures, module)
{ 
  if(isStructureAvailable(module))
  {
    var u = structures.filter(function (s)
    {
      switch(s.stattype)
      {
         	 case POWER_GEN:
           case RESEARCH_LAB:
           return s.modules < 1;
           case VTOL_FACTORY:  //useless if you make only light bodies.
           case FACTORY:
           return s.modules < 2;
	         default:
           debug("player[" + me + "] buildModules() fail.");
      }
    });
    if(u.length > 0)
    {
      buildAt(module, u[0].x, u[0].y, false, true);
	  }
  }
}
//--------------------------------------------------------------------------
//Add defenses to nearest defense zone, creates a new defense zone if built an oil derrick far from base.
//Calculates base radius to be defended.
//--------------------------------------------------------------------------
function eventStructureBuilt(structure)
{
  if(structure.stattype == DEFENSE) //structure.stattype != HQ && ( || structure.isSensor || structure.isCB || structure.isRadarDetector) 
  {	
    addDenfeseToZone(structure);
  }
  else if(structure.stattype == RESOURCE_EXTRACTOR)//Got oil resourse? Build defenses if it is far.
  {
    var defend_it = true;
    for(var a = 0; a < storage.defenseZones.length; a++)
    {
      var distance = distBetweenTwoPoints(structure.x, structure.y, storage.defenseZones[a].x, storage.defenseZones[a].y);
      var distancefromBase = distBetweenTwoPoints(structure.x, structure.y, startPositions[me].x, startPositions[me].y);
      if(distance < storage.baseRange || distancefromBase < storage.baseRange)
      {
        defend_it = false;
        break;
      }
    }
    if(defend_it)
    {
      storage.defenseZones.push(new DefenseZone(structure.x, structure.y));
    }
  }
  //Base area to be defended.
  var x = startPositions[me].x;
  var y = startPositions[me].y;
  var x1 = startPositions[me].x;
  var y1 = startPositions[me].y;
  for(var a = 0; a < temp.structuresList.length; a++)
  {
    switch(temp.structuresList[a].stattype)
    {
      case COMMAND_CONTROL:
      case CYBORG_FACTORY:
      case FACTORY:
      case HQ:
      case LASSAT:
      case POWER_GEN:
      case REARM_PAD:
      case REPAIR_FACILITY:
      case RESEARCH_LAB:
      case SAT_UPLINK:
      case VTOL_FACTORY:
      if(temp.structuresList[a].x < x)
      {
        x = temp.structuresList[a].x;
      }
      if(temp.structuresList[a].y < y)
      {
        y = temp.structuresList[a].y;
      }
      if(temp.structuresList[a].x > x1)
      {
        x1 = temp.structuresList[a].x;
      }
      if(temp.structuresList[a].y > y1)
      {
        y1 = temp.structuresList[a].y;
      }
      storage.baseRange = Math.ceil(Math.sqrt(Math.pow(x1-x, 2) + Math.pow(y1-y, 2))); //Base radius
      break;
      default:
      break;
    }
  }

  if(storage.baseRange < temp.startBaseRange)
  {
    storage.baseRange = temp.startBaseRange;
  }
}
//--------------------------------------------------------------------------
//Find oil resources without derricks built on them.
//--------------------------------------------------------------------------
function checkOilResource()
{
  return enumFeature(ALL_PLAYERS).filter(function(feature){ return (feature.stattype == OIL_RESOURCE); });
}
//--------------------------------------------------------------------------
//Sort array of objects by distance from base.
//--------------------------------------------------------------------------
function sort_NearestBase(a, b){
 return (distBetweenTwoPoints(startPositions[me].x, startPositions[me].y, a.x, a.y) - distBetweenTwoPoints(startPositions[me].x, startPositions[me].y, b.x, b.y));
}
//--------------------------------------------------------------------------
//Remove old tickets.
//--------------------------------------------------------------------------
function cleanBuildTickets()
{  
//  debug("AI[" + me + "] buildTickets " + storage.buildTickets.length);
//  storage.buildTickets.forEach(function (t, i){ debug("[" + i + "] truck id = " + t.truck.id + " structure " + t.structure + " (" + t.x + ", " + t.y + ")"); });
  storage.buildTickets = storage.buildTickets.filter(function (t)
  {
//    debug("Still alive id = " + t.id);
    for(var a = 0; a < temp.trucks.length; a++)//check if this truck still exists.
    {
 //     debug("trucks [" + a + "] " + temp.trucks[a].id);
      if(temp.trucks[a].id == t.truck.id)
      {
        return true;
      }
    }
    return false;
  });
//  debug("Still alive total " + storage.buildTickets.length);
//  storage.buildTickets.forEach(function (t, i){ debug("[" + i + "] truck id = " + t.truck.id + " structure " + t.structure + " (" + t.x + ", " + t.y + ")"); });
  storage.buildTickets = storage.buildTickets.filter(function (t)
  {
    for(var a = 0; a < temp.trucksIdle.length; a++)//check if this truck still busy.
    {
      if(temp.trucksIdle[a].id == t.truck.id)
      {
        return false;
      }
    }
    return true;
  });
//  debug("Filtered buildTickets " + storage.buildTickets.length);
//  storage.buildTickets.forEach(function (t, i){ debug("[" + i + "] truck id = " + t.truck.id + " structure " + t.structure + " (" + t.x + ", " + t.y + ")"); });
}


