//--------------------------------------------------------------------------
//Makes trucks, tanks and vtols.
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
//Make a truck.
//--------------------------------------------------------------------------
function makeTruck()
{
  if(temp.trucks.length < storage.trucksLimit) //make trucks if running low.
  {
    var bodylist = [
      "Body4ABT", // bug
      "Body1REC", // viper
    ];
    var proplist = [
      "hover01", // hover
      "wheeled01", // wheels
    ];
    return buildDroid(temp.idleFactory, "Truck", bodylist, proplist, null, null, "Spade1Mk1");
  }
  else if(componentAvailable("Body4ABT") && componentAvailable("hover01")) //recycle trucks one by one
  {
    var trucksOld = temp.trucksIdle.filter(function (d){ return d.body != "Body4ABT" || d.propulsion != "hover01"; });
    if(trucksOld.length > 0)
    {
       orderDroid(trucksOld[0], DORDER_RECYCLE);
    }
  }
  return false;
}
//--------------------------------------------------------------------------
//Make a sensor.
//--------------------------------------------------------------------------
function makeMobileSensor()
{
  if(storage.useTanks && temp.sensors.length < storage.sensors.min && (temp.mobartillery.length > 0 || temp.indirect.length > 0))
  {
    var bodylist = [ 
      "Body7ABT", //Retribution
      "Body6SUPP", //Panther
      "Body5REC",  //Cobra
      "Body1REC"  //Viper
    ];
    return buildDroid(temp.idleFactory, "Sensor", bodylist, storage.mobileArtilleryPropulsion, null, null, ["Sensor-WideSpec", "SensorTurret1Mk1"]);
  }
  else
  { //Replace old sensors for wide spectrum, one by one.
    var oldsensors = temp.sensors.filter(function (s){ return s.born < storage.WSTime; });
    if(oldsensors.length > 0)
    {
      orderDroid(oldsensors[0], DORDER_RECYCLE);
    }
  }
  return false;
}
//--------------------------------------------------------------------------
//Make a artillery.
//--------------------------------------------------------------------------
function makeMobileArtillery()
{
  if(storage.useTanks && temp.mobartillery.length < storage.mobartillery.max)
  {
    var MobileArtillery = temp.mobileArtilleryArmory.pick();
    if(MobileArtillery)
    {
      var bodylist = [ 
        "Body11ABT", //Python
        "Body5REC",  //Cobra
        "Body1REC"  //Viper
      ];
      return buildDroid(temp.idleFactory, "Mobile artillery", bodylist, storage.mobileArtilleryPropulsion, null, null, MobileArtillery);
    }
  }
  else
  {
    var mobileArtillery = temp.mobartillery.filter(function (d){ return temp.deprecated.indexOf(d.weapons[0].name) >= 0;});
    if(mobileArtillery.length > 0)
    {
      orderDroid(mobileArtillery[0], DORDER_RECYCLE);
    }
  }
  return false;
}
//--------------------------------------------------------------------------
//Make a tank.
//--------------------------------------------------------------------------
function makeTank()
{
  if(storage.useTanks && temp.tanks.length < storage.tanks.max) 
  {
    var weapon = temp.tankArmory.pick();
    if(weapon)
    {
      var bodylist;
      if(temp.tanks.length < storage.tanks.max*0.38)
      {
        bodylist = [ 
          "Body6SUPP", //Panther
          "Body5REC",  //Cobra
          "Body1REC"  //Viper
        ];
      }
      else if(temp.tanks.length < storage.tanks.max*0.08)
      {
        bodylist = [ 
          "Body2SUP", //Leopard
          "Body1REC"  //Viper
        ];
      }
      else
      {
        bodylist = 
        [ 
          "Body13SUP", //Wyvern
          "Body10MBT", //Vengeance
          "Body7ABT", //Retribution
          "Body3MBT", //Retaliation
          "Body9REC", //Tiger
          "Body11ABT", //Python
          "Body5REC",  //Cobra
          "Body1REC"  //Viper
        ];
      }
      temp.useDragon = !temp.useDragon;
      if(temp.useDragon && componentAvailable("Body14SUP"))
      {
        return buildDroid(temp.idleFactory, "Tank", "Body14SUP", storage.tankPropulsion, null, null, weapon, temp.tankArmory.pick());
      }
      else 
      {
        return buildDroid(temp.idleFactory, "Tank", bodylist, storage.tankPropulsion, null, null, weapon);
      }
    } 
  }
  return false;
}
//--------------------------------------------------------------------------
//Make droids once a factory is ready.
//--------------------------------------------------------------------------
function makeUnits()  
{
  var ret = false;
  for(var a = 0; a < temp.factories.length; a++)
  { //--------------------------------------------------------------------------
    if(structureIdle(temp.factories[a])){
      temp.idleFactory = temp.factories[a];
      if(temp.ProductionMan.run())
      {
        ret = true;
      }
    }
  }

  //VTOLS
  if(temp.vtols.length < storage.vtols.max)
  {
    for(var a = 0; a < temp.vtolFactories.length; a++)
    {
      if(structureIdle(temp.vtolFactories[a]))
      {
        var weapon = temp.vtolArmory.pick();
        if(weapon)
        {
          var bodylist = [
            "Body13SUP", //Wyvern
            "Body10MBT", //Vengeance
            "Body7ABT", //Retribution
            "Body3MBT", //Retaliation
            "Body9REC", //Tiger
            "Body11ABT", //Python
            "Body2SUP", //Leopard
            "Body4ABT", // bug
            "Body1REC", // viper
          ];
          if(buildDroid(temp.vtolFactories[a], "VTOL", bodylist, "V-Tol", null, null, weapon))
          {
            ret = true;
          }
        }
      }
    }
  }
  return ret;
}
