
//Game start settings

//Executes taks (functions) in sequence each time .run() is called. Stops atempting to call the functions once any returns true or all have been called.
function TaskManager()
{
  this.tasks = arguments; //Array of functions that must return true or false;
  this.step = 0;
  
  this.run = function()
  {
    var result = false;
    for(var a = 0; a < this.tasks.length && !result; a++)
    {
      result = this.tasks[this.step].call();    
      this.step++;
      if(!(this.step < this.tasks.length))
      {
        this.step = 0;
      }
    }
    return result;
  };  
}

var storage; //Stores data that need to be saved.
var temp; //Stores data that do not need to be saved.

function eventStartLevel()
{
  var onlyVtols = true;
  var onlyHover = false;
  for(var a = 0; a < playerData.length; a++)
  {
    if(a != me)
    {
      if(propulsionCanReach("hover01", startPositions[me].x, startPositions[me].y, startPositions[a].x, startPositions[a].y))
      {
        onlyVtols = false;
      }
      if(!propulsionCanReach("wheeled01", startPositions[me].x, startPositions[me].y, startPositions[a].x, startPositions[a].y))
      {
        onlyHover = true;
      }
    }
  }
  eventGameLoaded()
  if(!storage)   //Savegame data
  {
	storage = {
	  TaskPower:200,  //Minimum power needed to allow trucks to build something and do research.
	  buildTickets: [],
	  WSTime: 0, //Gamplay time when the Wide Spectrum Sensor became available.
	  techTree: [],  //Tech tree to be researched.
	  offensive: false, //false - defend the base, true - attack enemies.
	  baseRange: temp.startBaseRange,  //Base radius to be defended.
	  droidsBaseLimit: getDroidLimit(me),
	  droidsAttackMin: 0,
	  tanks: new Squad(),
	  sensors: new Squad(),
	  mobartillery: new Squad(),
	  vtols: new Squad(),
	  heavyVtols: false,
	  trucksLimit: 10,    
	  tankPropulsion: onlyHover ? ["hover01"] : ["tracked01", "HalfTrack", "wheeled01"],
	  useTanks: !onlyVtols,
	  mobileArtilleryPropulsion: onlyHover ? ["hover01"] : ["HalfTrack", "wheeled01"],
	  personality: ChoosePersonality(),
	  beaconReceived: { time: 0, x: 0, y: 0},
	  beaconSent: 0,
	  lastRequest: 0,
	  chatOn: true,
	  PlasmaHeavyBug: false //Workaround for a bug that allows to make droids with PlasmaHeavy before research is done.
	};
  }
  storage.defenseZones = [new DefenseZone(startPositions[me].x, startPositions[me].y)]; //Defense zones. [0] = base, [1],[2]... oil resources to defend.
  storage.defenseZones[0].limits.at = (storage.useTanks) ? 8 : 0;
  storage.defenseZones[0].limits.indirect = (storage.useTanks) ? 5 : 0;
  storage.defenseZones[0].limits.artillery = 25;
  storage.defenseZones[0].limits.aa = 15;
  
  setResearch();
  var ttime;
  storage.sensors.min = 3;
  storage.droidsBaseLimit -= storage.trucksLimit + storage.sensors.min;  //137
  var droidsBaseFactor = 1;
  switch(difficulty)
  {
    case EASY:
    ttime = 3000;
    droidsBaseFactor = 0.4; //works with fewer droids.
    break;
    case MEDIUM:
    ttime = 2000;
    droidsBaseFactor = 0.6;
    break;
    case HARD:
    ttime = 1500;
    droidsBaseFactor = 0.8;
    break;
    case INSANE: //works with more droids.
    ttime = 1000;
    break;
  }
  storage.droidsBaseLimit = Math.round(storage.droidsBaseLimit*droidsBaseFactor);
  storage.droidsAttackMin = Math.round(storage.droidsBaseLimit*0.1);
  storage.mobartillery.max = Math.round(storage.droidsBaseLimit*0.35);
  storage.vtols.max = Math.round(storage.droidsBaseLimit*0.20);
  storage.tanks.max = Math.round(storage.droidsBaseLimit*0.45);
  storage.tanks.min = Math.round(storage.tanks.max*0.2);
  if(onlyVtols)
  {
    storage.vtols.max = storage.tanks.max;
  }
  setTimer("doTasks", ttime);
}
//--------------------------------------------------------------------------
// Recreates objects that are not stored at save games.
//--------------------------------------------------------------------------
function eventGameLoaded()
{
  temp = function(){};
  temp.myPower = playerPower(me);
  temp.TaskMan = new TaskManager(buildBase, makeUnits, doResearch);
  temp.BaseMan = new TaskManager(buildBasePower, buildBaseTech, buildBaseFactories, buildDefenses);
  temp.ProductionMan = new TaskManager(makeTank, makeTruck, makeMobileSensor, makeMobileArtillery);
  temp.defenseZoneMan = new TaskManager(weNeedSensor, weNeedAT, weNeedIndirect, weNeedArtillery, weNeedAA);
  temp.tankArmory = new Armory();
  temp.vtolArmory = new Armory();
  temp.indirectDefensesArmory = new Armory();
  temp.ATDefensesArmory = new Armory();
  temp.AADefensesArmory = new Armory();
  temp.ArtilleryArmory = new Armory();
  temp.mobileArtilleryArmory = new Armory();
  temp.idleFactory = 0;
  temp.useDragon = false;
  temp.startBaseRange = 15;
  temp.artilleryRange = 4480;
  temp.lossTolerance = 0.6;
  temp.boundWidth = mapWidth - 4;
  temp.boundHeight = mapHeight - 4;
  eventResearched();
}
//--------------------------------------------------------------------------
// Battle field snapshot.
//--------------------------------------------------------------------------
function BFSnapShot()
{
  temp.myPower = playerPower(me) - queuedPower(me);
  temp.alldroids = enumDroid();
  temp.tanks = temp.alldroids.filter(function (d){ if(d.droidType == DROID_WEAPON){ return !d.isVTOL && !d.hasIndirect; } return false; });
  temp.indirect = temp.alldroids.filter(function (d){ if(d.droidType == DROID_WEAPON){ return d.hasIndirect && d.range < temp.artilleryRange; } return false; });
  temp.sensors = temp.alldroids.filter(function (d){ return d.isSensor; });
  temp.mobartillery = temp.alldroids.filter(function (d){ if(d.droidType == DROID_WEAPON){ return d.hasIndirect && d.range > temp.artilleryRange;} return false; });
  temp.vtols = temp.alldroids.filter(function(d){return d.isVTOL; });
  temp.trucks = temp.alldroids.filter(function (d){ return d.droidType == DROID_CONSTRUCT; });
  temp.trucksIdle = temp.trucks.filter(function (d){ return d.order == 0 && d.action == 0; });
}
//--------------------------------------------------------------------------
// Base structures snapshot.
//--------------------------------------------------------------------------
function BaseSnapShot()
{ 
  temp.structuresList = enumStruct(me);
  temp.generators = temp.structuresList.filter(function (s){ return s.stattype == POWER_GEN; });
  temp.oilDerricks = temp.structuresList.filter(function (s){ return s.stattype == RESOURCE_EXTRACTOR; });
  temp.labs = temp.structuresList.filter(function (s){ return s.stattype == RESEARCH_LAB; });
  temp.factories = temp.structuresList.filter(function (s){ return s.stattype == FACTORY; });
  temp.vtolFactories = temp.structuresList.filter(function (s){ return s.stattype == VTOL_FACTORY; });
  temp.vtolPads = temp.structuresList.filter(function (s){ return s.stattype == REARM_PAD; });
}
//--------------------------------------------------------------------------
// Do task like building base structures, defenses, artillery and 
//--------------------------------------------------------------------------
function doTasks() //Main loop
{
  BFSnapShot();
  BaseSnapShot();
  cleanBuildTickets();
  oilGiveUp();
  oilHunt();  
  warTactic();
  if(storage.TaskPower > 0)
  {
    storage.TaskPower -= 1;
  }
  if(temp.myPower > storage.TaskPower)
  {
    temp.TaskMan.run();
  }
}
