//Zones for placing  defenses

function DefenseZone(X, Y) //Create a defense zone on a area or position;
{
  this.x = X;
  this.y = Y;
	this.defenses = newGroup();
	this.limits = 
  {
	  at: (storage.useTanks) ? 4 : 0,
	  indirect: (storage.useTanks) ? 4 : 0,
	  artillery: 4,
    aa: 4
	};
}

//Determine which defense zone this defense belongs.
function addDenfeseToZone(structure)
{
  var index = indexOfNearest(storage.defenseZones, structure.x, structure.y);
  groupAdd(storage.defenseZones[index].defenses , structure);
}
//--------------------------------------------------------------------------
// Build defenses at the zones. Demolish old defense.
//--------------------------------------------------------------------------
function buildDefenses()
{
  for(temp.currentZone = 0; temp.currentZone < storage.defenseZones.length; temp.currentZone++)
  {
    temp.structures = enumGroup(storage.defenseZones[temp.currentZone].defenses); //Avoid using enum more than one time.
    if(!temp.structures)
    {
      temp.structures = [];
    }
    temp.needed = undefined;
    temp.defenseZoneMan.run();
    if(temp.needed)
    {
      return !buildAt(temp.needed, storage.defenseZones[temp.currentZone].x, storage.defenseZones[temp.currentZone].y, true, (temp.currentZone == 0));
    }
    temp.needed = hasDeprecated();
    if(temp.needed)
    {
      return DemolishIt(temp.needed);
    }
  }
  return false;
}
//--------------------------------------------------------------------------
// Returns which sensor a zone needs.
//--------------------------------------------------------------------------
function weNeedSensor()
{
  var needSensor = true;
  var needCB = true;
  var needRadarDetector = true;
  for(var a = 0; a < temp.structures.length; a++)
  {
    if(temp.structures[a].isSensor)
    {
      needSensor = false;
    }
    if(temp.structures[a].isCB)
    {
      needCB = false;
    }
    if(temp.structures[a].isRadarDetector)
    {
      needRadarDetector = false;
    }
  }
  if(needSensor)
  {
    if(isStructureAvailable("Sys-SensoTowerWS"))
    {
      temp.needed = "Sys-SensoTowerWS";
      return true;
    }
    if(isStructureAvailable("Sys-SensoTower02"))
    {
      temp.needed = "Sys-SensoTower02";
      return true;
    }
  }
  if(needCB)
  {
    if(isStructureAvailable("Sys-CB-Tower01"))
    {
      temp.needed = "Sys-CB-Tower01";
      return true;
    }
  }
  if(needRadarDetector)
  {
    if(isStructureAvailable("Sys-RadarDetector01"))
    {
      temp.needed = "Sys-RadarDetector01";
      return true;
    }
  }
  return false;
}
//--------------------------------------------------------------------------
// Returns which AT defense a zone needs.
//--------------------------------------------------------------------------  
function weNeedAT() //AT
{
  if(temp.structures.filter(function (d){ return d.canHitGround && !d.hasIndirect; }).length < storage.defenseZones[temp.currentZone].limits.at)
  {
    temp.needed = temp.ATDefensesArmory.pick();
    if(temp.needed)
    {  
      return true;
    }
  }
  return false;
}
//--------------------------------------------------------------------------
// Returns which indirect defense a zone needs.
//--------------------------------------------------------------------------  
function weNeedIndirect()
{
  if(temp.structures.filter(function (d){ return d.range < temp.artilleryRange && d.hasIndirect; }).length < storage.defenseZones[temp.currentZone].limits.indirect)
  {
    temp.needed = temp.indirectDefensesArmory.pick();
    if(temp.needed)
    {  
      return true;
    }
  }
  return false;
}
//--------------------------------------------------------------------------
// Returns which artillery defense a zone needs.
//--------------------------------------------------------------------------
function weNeedArtillery()
{
  if(temp.structures.filter(function (d){ return d.range > temp.artilleryRange && d.hasIndirect; }).length < storage.defenseZones[temp.currentZone].limits.artillery)
  {
    temp.needed = temp.ArtilleryArmory.pick();
    if(temp.needed)
    {  
      return true;
    }
  }
  return false;
}
//--------------------------------------------------------------------------
// Returns which AA defense a zone needs.
//--------------------------------------------------------------------------  
function weNeedAA() //AA
{
  if(temp.structures.filter(function (d){ return !d.canHitGround && d.canHitAir; }).length < storage.defenseZones[temp.currentZone].limits.aa)
  {
    temp.needed = temp.AADefensesArmory.pick();
    if(temp.needed)
    {  
      return true;
    }
  }
  return false;
}
//--------------------------------------------------------------------------
//Removes old defense guns and sensor if Wide Spectrum sensor available.
//--------------------------------------------------------------------------
function hasDeprecated()
{
  for(var c = 0; c < temp.structures.length; c++)
  {
    if((temp.structures[c].weapons[0] && temp.deprecated.indexOf(temp.structures[c].weapons[0].name) >= 0) || (temp.structures[c].isSensor && temp.structures[c].born < storage.WSTime))
    {
      return temp.structures[c];
    }
  }  
}
